@@ -39,7 +39,6 @@
 #include <libavutil/intreadwrite.h>
 #include <libavutil/intfloat.h>
 
-#include "config.h"
 #include "ao.h"
 #include "internal.h"
 #include "audio/format.h"
@@ -73,7 +72,7 @@ struct priv {
     AudioStreamBasicDescription original_asbd;
 
     // Output s16 physical format, float32 virtual format, ac3/dts mpv format
-    int spdif_hack;
+    bool spdif_hack;
 
     bool changed_mixing;
 
@@ -455,7 +454,7 @@ const struct ao_driver audio_out_coreaudio_exclusive = {
     .uninit    = uninit,
     .init      = init,
     .reset     = audio_pause,
-    .resume    = audio_resume,
+    .start     = audio_resume,
     .list_devs = ca_get_device_list,
     .priv_size = sizeof(struct priv),
     .priv_defaults = &(const struct priv){
@@ -465,7 +464,7 @@ const struct ao_driver audio_out_coreaudio_exclusive = {
         .changed_mixing = false,
     },
     .options = (const struct m_option[]){
-        OPT_FLAG("spdif-hack", spdif_hack, 0),
+        {"spdif-hack", OPT_BOOL(spdif_hack)},
         {0}
     },
     .options_prefix = "coreaudio",
