@@ -36,9 +36,7 @@
 #include <libavutil/display.h>
 #include <libavutil/opt.h>
 
-#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(56, 43, 100)
 #include <libavutil/dovi_meta.h>
-#endif
 
 #include "audio/chmap_avchannel.h"
 
@@ -78,14 +76,14 @@ struct demux_lavf_opts {
     int probescore;
     float analyzeduration;
     int buffersize;
-    int allow_mimetype;
+    bool allow_mimetype;
     char *format;
     char **avopts;
-    int hacks;
+    bool hacks;
     char *sub_cp;
     int rtsp_transport;
     int linearize_ts;
-    int propagate_opts;
+    bool propagate_opts;
 };
 
 const struct m_sub_options demux_lavf_conf = {
@@ -98,10 +96,10 @@ const struct m_sub_options demux_lavf_conf = {
          M_RANGE(0, 3600)},
         {"demuxer-lavf-buffersize", OPT_INT(buffersize),
          M_RANGE(1, 10 * 1024 * 1024), OPTDEF_INT(BIO_BUFFER_SIZE)},
-        {"demuxer-lavf-allow-mimetype", OPT_FLAG(allow_mimetype)},
+        {"demuxer-lavf-allow-mimetype", OPT_BOOL(allow_mimetype)},
         {"demuxer-lavf-probescore", OPT_INT(probescore),
          M_RANGE(1, AVPROBE_SCORE_MAX)},
-        {"demuxer-lavf-hacks", OPT_FLAG(hacks)},
+        {"demuxer-lavf-hacks", OPT_BOOL(hacks)},
         {"demuxer-lavf-o", OPT_KEYVALUELIST(avopts)},
         {"sub-codepage", OPT_STRING(sub_cp)},
         {"rtsp-transport", OPT_CHOICE(rtsp_transport,
@@ -112,14 +110,14 @@ const struct m_sub_options demux_lavf_conf = {
             {"udp_multicast", 4})},
         {"demuxer-lavf-linearize-timestamps", OPT_CHOICE(linearize_ts,
             {"no", 0}, {"auto", -1}, {"yes", 1})},
-        {"demuxer-lavf-propagate-opts", OPT_FLAG(propagate_opts)},
+        {"demuxer-lavf-propagate-opts", OPT_BOOL(propagate_opts)},
         {0}
     },
     .size = sizeof(struct demux_lavf_opts),
     .defaults = &(const struct demux_lavf_opts){
         .probeinfo = -1,
-        .allow_mimetype = 1,
-        .hacks = 1,
+        .allow_mimetype = true,
+        .hacks = true,
         // AVPROBE_SCORE_MAX/4 + 1 is the "recommended" limit. Below that, the
         // user is supposed to retry with larger probe sizes until a higher
         // value is reached.
@@ -127,7 +125,7 @@ const struct m_sub_options demux_lavf_conf = {
         .sub_cp = "auto",
         .rtsp_transport = 2,
         .linearize_ts = -1,
-        .propagate_opts = 1,
+        .propagate_opts = true,
     },
 };
 
@@ -225,6 +223,12 @@ struct stream_info {
     double ts_offset;
 };
 
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(59, 10, 100)
+    #define HAVE_IO_CLOSE2 1
+#else
+    #define HAVE_IO_CLOSE2 0
+#endif
+
 typedef struct lavf_priv {
     struct stream *stream;
     bool own_stream;
@@ -258,7 +262,11 @@ typedef struct lavf_priv {
     int num_nested;
     int (*default_io_open)(struct AVFormatContext *s, AVIOContext **pb,
                            const char *url, int flags, AVDictionary **options);
+#if HAVE_IO_CLOSE2
+    int (*default_io_close2)(struct AVFormatContext *s, AVIOContext *pb);
+#else
     void (*default_io_close)(struct AVFormatContext *s, AVIOContext *pb);
+#endif
 } lavf_priv_t;
 
 static void update_read_stats(struct demuxer *demuxer)
@@ -458,11 +466,16 @@ static int lavf_check_file(demuxer_t *demuxer, enum demux_check check)
         }
     }
 
+    // HLS streams seems to be not well tagged, so matching mime type is not
+    // enough. Strip URL parameters and match extension.
+    bstr ext = bstr_get_ext(bstr_split(bstr0(priv->filename), "?#", NULL));
     AVProbeData avpd = {
-        // Disable file-extension matching with normal checks
-        .filename = check <= DEMUX_CHECK_REQUEST ? priv->filename : "",
+        // Disable file-extension matching with normal checks, except for HLS
+        .filename = !bstrcasecmp0(ext, "m3u8") || !bstrcasecmp0(ext, "m3u") ||
+                    check <= DEMUX_CHECK_REQUEST ? priv->filename : "",
         .buf_size = 0,
         .buf = av_mallocz(PROBE_BUF_SIZE + AV_INPUT_BUFFER_PADDING_SIZE),
+        .mime_type = lavfdopts->allow_mimetype ? mime_type : NULL,
     };
     if (!avpd.buf)
         return -1;
@@ -745,14 +758,12 @@ static void handle_new_stream(demuxer_t *demuxer, int i)
                 sh->codec->rotate = (((int)(-r) % 360) + 360) % 360;
         }
 
-#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(56, 43, 100)
         if ((sd = av_stream_get_side_data(st, AV_PKT_DATA_DOVI_CONF, NULL))) {
             const AVDOVIDecoderConfigurationRecord *cfg = (void *) sd;
             MP_VERBOSE(demuxer, "Found Dolby Vision config record: profile "
                        "%d level %d\n", cfg->dv_profile, cfg->dv_level);
             av_format_inject_global_side_data(avfc);
         }
-#endif
 
         // This also applies to vfw-muxed mkv, but we can't detect these easily.
         sh->codec->avi_dts = matches_avinputformat_name(priv, "avi");
@@ -836,6 +847,9 @@ static void handle_new_stream(demuxer_t *demuxer, int i)
         if (lang && lang->value && strcmp(lang->value, "und") != 0)
             sh->lang = talloc_strdup(sh, lang->value);
         sh->hls_bitrate = dict_get_decimal(st->metadata, "variant_bitrate", 0);
+        AVProgram *prog = av_find_program_from_stream(avfc, NULL, i);
+        if (prog)
+            sh->program_id = prog->id;
         sh->missing_timestamps = !!(priv->avif_flags & AVFMT_NOTIMESTAMPS);
         mp_tags_copy_from_av_dictionary(sh->tags, st->metadata);
         demux_add_sh_stream(demuxer, sh);
@@ -930,7 +944,11 @@ static int nested_io_open(struct AVFormatContext *s, AVIOContext **pb,
     return r;
 }
 
+#if HAVE_IO_CLOSE2
+static int nested_io_close2(struct AVFormatContext *s, AVIOContext *pb)
+#else
 static void nested_io_close(struct AVFormatContext *s, AVIOContext *pb)
+#endif
 {
     struct demuxer *demuxer = s->opaque;
     lavf_priv_t *priv = demuxer->priv;
@@ -942,16 +960,21 @@ static void nested_io_close(struct AVFormatContext *s, AVIOContext *pb)
         }
     }
 
-
+#if HAVE_IO_CLOSE2
+    return priv->default_io_close2(s, pb);
+#else
     priv->default_io_close(s, pb);
+#endif
 }
 
 static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
 {
-    AVFormatContext *avfc;
+    AVFormatContext *avfc = NULL;
     AVDictionaryEntry *t = NULL;
     float analyze_duration = 0;
     lavf_priv_t *priv = talloc_zero(NULL, lavf_priv_t);
+    AVDictionary *dopts = NULL;
+
     demuxer->priv = priv;
     priv->stream = demuxer->stream;
 
@@ -965,11 +988,11 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
                        &priv->mf_fps);
 
     if (lavf_check_file(demuxer, check) < 0)
-        return -1;
+        goto fail;
 
     avfc = avformat_alloc_context();
     if (!avfc)
-        return -1;
+        goto fail;
 
     if (index_mode != 1)
         avfc->flags |= AVFMT_FLAG_IGNIDX;
@@ -991,8 +1014,6 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
                    "analyzeduration to %f\n", analyze_duration);
     }
 
-    AVDictionary *dopts = NULL;
-
     if ((priv->avif_flags & AVFMT_NOFILE) || priv->format_hack.no_stream) {
         mp_setup_av_network_options(&dopts, priv->avif->name,
                                     demuxer->global, demuxer->log);
@@ -1001,12 +1022,12 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
     } else {
         void *buffer = av_malloc(lavfdopts->buffersize);
         if (!buffer)
-            return -1;
+            goto fail;
         priv->pb = avio_alloc_context(buffer, lavfdopts->buffersize, 0,
                                       demuxer, mp_read, NULL, mp_seek);
         if (!priv->pb) {
             av_free(buffer);
-            return -1;
+            goto fail;
         }
         priv->pb->read_seek = mp_read_seek;
         priv->pb->seekable = demuxer->seekable ? AVIO_SEEKABLE_NORMAL : 0;
@@ -1041,9 +1062,14 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
     avfc->opaque = demuxer;
     if (demuxer->access_references) {
         priv->default_io_open = avfc->io_open;
-        priv->default_io_close = avfc->io_close;
         avfc->io_open = nested_io_open;
+#if HAVE_IO_CLOSE2
+        priv->default_io_close2 = avfc->io_close2;
+        avfc->io_close2 = nested_io_close2;
+#else
+        priv->default_io_close = avfc->io_close;
         avfc->io_close = nested_io_close;
+#endif
     } else {
         avfc->io_open = block_io_open;
     }
@@ -1051,8 +1077,8 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
     mp_set_avdict(&dopts, lavfdopts->avopts);
 
     if (av_dict_copy(&priv->av_opts, dopts, 0) < 0) {
-        av_dict_free(&dopts);
-        return -1;
+        MP_ERR(demuxer, "av_dict_copy() failed\n");
+        goto fail;
     }
 
     if (priv->format_hack.readall_on_no_streamseek && priv->pb &&
@@ -1071,8 +1097,7 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
 
     if (avformat_open_input(&avfc, priv->filename, priv->avif, &dopts) < 0) {
         MP_ERR(demuxer, "avformat_open_input() failed\n");
-        av_dict_free(&dopts);
-        return -1;
+        goto fail;
     }
 
     mp_avdict_print_unset(demuxer->log, MSGL_V, dopts);
@@ -1090,7 +1115,7 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
     if (probeinfo) {
         if (avformat_find_stream_info(avfc, NULL) < 0) {
             MP_ERR(demuxer, "av_find_stream_info() failed\n");
-            return -1;
+            goto fail;
         }
 
         MP_VERBOSE(demuxer, "avformat_find_stream_info() finished after %"PRId64
@@ -1170,6 +1195,13 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
     }
 
     return 0;
+
+fail:
+    if (!priv->avfc)
+        avformat_free_context(avfc);
+    av_dict_free(&dopts);
+
+    return -1;
 }
 
 static bool demux_lavf_read_packet(struct demuxer *demux,
@@ -1311,10 +1343,11 @@ static void demux_seek_lavf(demuxer_t *demuxer, double seek_pts, int flags)
     if (priv->pcm_seek_hack && !priv->pcm_seek_hack_packet_size) {
         // This might for example be the initial seek. Fuck it up like the
         // bullshit it is.
-        AVPacket pkt = {0};
-        if (av_read_frame(priv->avfc, &pkt) >= 0)
-            priv->pcm_seek_hack_packet_size = pkt.size;
-        av_packet_unref(&pkt);
+        AVPacket *pkt = av_packet_alloc();
+        MP_HANDLE_OOM(pkt);
+        if (av_read_frame(priv->avfc, pkt) >= 0)
+            priv->pcm_seek_hack_packet_size = pkt->size;
+        av_packet_free(&pkt);
         add_new_streams(demuxer);
     }
     if (priv->pcm_seek_hack && priv->pcm_seek_hack_packet_size &&
