@@ -29,8 +29,6 @@
 #include <libavutil/rational.h>
 #include <libavutil/cpu.h>
 
-#include "config.h"
-
 #include "common/msg.h"
 #include "options/m_option.h"
 #include "options/path.h"
@@ -111,25 +109,25 @@ struct script_driver {
 
 struct mpvs_fmt {
     VSPresetFormat vs;
-    int bits, cw, ch;
+    int bits, xs, ys;
 };
 
 static const struct mpvs_fmt mpvs_fmt_table[] = {
-    {pfYUV420P8,  8,  2, 2},
-    {pfYUV420P9,  9,  2, 2},
-    {pfYUV420P10, 10, 2, 2},
-    {pfYUV420P16, 16, 2, 2},
-    {pfYUV422P8,  8,  2, 1},
-    {pfYUV422P9,  9,  2, 1},
-    {pfYUV422P10, 10, 2, 1},
-    {pfYUV422P16, 16, 2, 1},
-    {pfYUV410P8,  8,  4, 4},
-    {pfYUV411P8,  8,  4, 1},
-    {pfYUV440P8,  8,  1, 2},
-    {pfYUV444P8,  8,  1, 1},
-    {pfYUV444P9,  9,  1, 1},
-    {pfYUV444P10, 10, 1, 1},
-    {pfYUV444P16, 16, 1, 1},
+    {pfYUV420P8,  8,  1, 1},
+    {pfYUV420P9,  9,  1, 1},
+    {pfYUV420P10, 10, 1, 1},
+    {pfYUV420P16, 16, 1, 1},
+    {pfYUV422P8,  8,  1, 0},
+    {pfYUV422P9,  9,  1, 0},
+    {pfYUV422P10, 10, 1, 0},
+    {pfYUV422P16, 16, 1, 0},
+    {pfYUV410P8,  8,  2, 2},
+    {pfYUV411P8,  8,  2, 0},
+    {pfYUV440P8,  8,  0, 1},
+    {pfYUV444P8,  8,  0, 0},
+    {pfYUV444P9,  9,  0, 0},
+    {pfYUV444P10, 10, 0, 0},
+    {pfYUV444P16, 16, 0, 0},
     {pfNone}
 };
 
@@ -140,7 +138,7 @@ static bool compare_fmt(int imgfmt, const struct mpvs_fmt *vs)
         return false;
     if (rfmt.component_pad > 0)
         return false;
-    if (rfmt.chroma_w != vs->cw || rfmt.chroma_h != vs->ch)
+    if (rfmt.chroma_xs != vs->xs || rfmt.chroma_ys != vs->ys)
         return false;
     if (rfmt.component_size * 8 + rfmt.component_pad != vs->bits)
         return false;
@@ -800,10 +798,11 @@ error:
 
 #define OPT_BASE_STRUCT struct vapoursynth_opts
 static const m_option_t vf_opts_fields[] = {
-    OPT_STRING("file", file, M_OPT_FILE),
-    OPT_INTRANGE("buffered-frames", maxbuffer, 0, 1, 9999, OPTDEF_INT(4)),
-    OPT_CHOICE_OR_INT("concurrent-frames", maxrequests, 0, 1, 99,
-                      ({"auto", -1}), OPTDEF_INT(-1)),
+    {"file", OPT_STRING(file), .flags = M_OPT_FILE},
+    {"buffered-frames", OPT_INT(maxbuffer), M_RANGE(1, 9999),
+        OPTDEF_INT(4)},
+    {"concurrent-frames", OPT_CHOICE(maxrequests, {"auto", -1}),
+        M_RANGE(1, 99), OPTDEF_INT(-1)},
     {0}
 };
 
