@@ -34,12 +34,12 @@ static int d3d11_validate_adapter(struct mp_log *log,
 struct d3d11_opts {
     int feature_level;
     int warp;
-    int flip;
+    bool flip;
     int sync_interval;
     char *adapter_name;
     int output_format;
     int color_space;
-    int exclusive_fs;
+    bool exclusive_fs;
 };
 
 #define OPT_BASE_STRUCT struct d3d11_opts
@@ -59,7 +59,7 @@ const struct m_sub_options d3d11_conf = {
             {"9_3", D3D_FEATURE_LEVEL_9_3},
             {"9_2", D3D_FEATURE_LEVEL_9_2},
             {"9_1", D3D_FEATURE_LEVEL_9_1})},
-        {"d3d11-flip", OPT_FLAG(flip)},
+        {"d3d11-flip", OPT_BOOL(flip)},
         {"d3d11-sync-interval", OPT_INT(sync_interval), M_RANGE(0, 4)},
         {"d3d11-adapter", OPT_STRING_VALIDATE(adapter_name,
                                               d3d11_validate_adapter)},
@@ -75,18 +75,17 @@ const struct m_sub_options d3d11_conf = {
             {"linear",  DXGI_COLOR_SPACE_RGB_FULL_G10_NONE_P709},
             {"pq",      DXGI_COLOR_SPACE_RGB_FULL_G2084_NONE_P2020},
             {"bt.2020", DXGI_COLOR_SPACE_RGB_FULL_G22_NONE_P2020})},
-        {"d3d11-exclusive-fs", OPT_FLAG(exclusive_fs)},
+        {"d3d11-exclusive-fs", OPT_BOOL(exclusive_fs)},
         {0}
     },
     .defaults = &(const struct d3d11_opts) {
         .feature_level = D3D_FEATURE_LEVEL_12_1,
         .warp = -1,
-        .flip = 1,
+        .flip = true,
         .sync_interval = 1,
         .adapter_name = NULL,
         .output_format = DXGI_FORMAT_UNKNOWN,
         .color_space = -1,
-        .exclusive_fs = 0,
     },
     .size = sizeof(struct d3d11_opts)
 };
